#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include <stdio.h>
#include <termios.h>
#include <string.h>
#include <stdbool.h>
#include <stdlib.h>

// Folgende zwei Makros von
// http://stackoverflow.com/questions/33025599/move-the-cursor-in-a-c-program
#define cursorforward(x) printf("\033[%dC", (x))
#define cursorbackward(x) printf("\033[%dD", (x))

#define MAX_LEN 80

int main (int argc, char *argv[]) {

  // initialize terminal
  // source: answer 1 in
  // http://stackoverflow.com/questions/32809983/how-do-i-use-the-ioctl-system-call-to-disable-keypress-echo-and-buffering
  struct termios old, new;
  tcgetattr(STDIN_FILENO, &old);          // get current settings
  new = old;                              // create a backup
  new.c_lflag &= ~(ICANON | ECHO);        // disable line buffering and feedback
  tcsetattr(STDIN_FILENO, TCSANOW, &new); // set our new config


  char buffer[81];
  if (argc==2)
    strncpy (buffer, argv[1], 80);
  else
    buffer[0] = 0;
  char len = strlen(buffer);
  int pos=0;
  printf ("%s", buffer);
  cursorbackward (len);

  char c;
  for (;;) {
    c = getchar ();
    // DEBUG
    //
    //printf ("ASCII: %d\n", c);
    //continue;
    
    if (c==27) {
      // Esc-Sequenz verarbeiten
      getchar (); c = getchar ();
      switch (c) {
        case 'D':   // Esc, [, D = cursor-left
          if (pos > 0)   { cursorbackward (1); pos--; }
          break;
        case 'C':   // ESC, [, C = cursor-right
          if (pos < len) { cursorforward  (1); pos++; }
          break;
        default: break;
      }
    } else {
      // normales Zeichen verarbeiten
      switch (c) {
        case '\n':
          printf ("\n");
          fprintf (stderr, "%s\n", buffer);       // write string to stderr
          tcsetattr(STDIN_FILENO, TCSANOW, &old); // restore old config
          exit (0);
        case '\t':
          // tab = ein Zeichen einfuegen
          if (len < MAX_LEN) {
            for (int i = len+1; i >= pos; i--) buffer[i+1] = buffer[i];
            buffer[pos] = ' ';
            len++;
            printf ("%s", buffer+pos);
            cursorbackward (len-pos);
          }
          break;
        case 127:
          // delete = ein Zeichen loeschen
          if (pos > 0) {
            for (int i = pos-1; i < len; i++) buffer[i] = buffer[i+1];
            len--; pos--;
            cursorbackward (1);
            printf ("%s ", buffer+pos);
            cursorbackward (len-pos+1);
          }
          break;
        default:
          if (pos < MAX_LEN) {
            printf ("%c", c); buffer[pos++] = c;
            if (pos>len) len++;
          }
          break;
      }
    }
  }
}

